''' User Account and Roles Models '''

import bcrypt
import enum
import uuid

from collections import namedtuple
from datetime import datetime
from sqlalchemy import (
    Column,
    Date,
    Enum,
    Integer,
    Text,
    )

from .meta import Base


class Role(enum.Enum):
    ''' roles of user accounts '''

    NEW = 'new'
    USER = 'user'
    PURCHASER = 'purchaser'
    ADMIN = 'admin'
    INACTIVE = 'inactive'

    @property
    def principal(self):
        ''' returns the principal identifier of the role '''
        return 'role:' + self.value.lower()


class User(Base):
    ''' A user of the application '''

    __tablename__ = 'users'

    id = Column(Integer, primary_key=True)

    user_name = Column(Text, nullable=False, unique=True)
    first_name = Column(Text, nullable=False)
    last_name = Column(Text, nullable=False)
    email = Column(Text, nullable=False, unique=True)
    password_hash = Column(Text, nullable=False)
    role = Column(Enum(Role), nullable=False)
    password_reset = Column(Text, nullable=False, default='')
    date_created = Column(Date, nullable=False, default=datetime.utcnow)

    @property
    def principal(self):
        ''' returns the principal identifier for the user '''
        return 'user:' + self.user_name

    @property
    def role_principals(self):
        ''' returns the principal identifiers for the user's role '''
        principals = [self.role.principal]
        if self.role is Role.PURCHASER:
            # a purchaser is also a user
            principals.append(Role.USER.principal)
        elif self.role is Role.ADMIN:
            # an admin is also a purchaser and a user
            principals.append(Role.USER.principal)
            principals.append(Role.PURCHASER.principal)
        return principals

    @property
    def is_active(self):
        ''' check if it is an active user account '''
        return self.role in (Role.USER, Role.PURCHASER, Role.ADMIN)

    def set_password(self, password):
        ''' hashes a new password '''
        pwhash = bcrypt.hashpw(password.encode('utf8'), bcrypt.gensalt())
        self.password_hash = pwhash.decode('utf8')

    def check_password(self, password):
        ''' compares a password with a stored password hash '''
        if self.password_hash:
            expected_hash = self.password_hash.encode('utf8')
            return bcrypt.checkpw(password.encode('utf8'), expected_hash)
        return False

    def generate_password_token(self):
        ''' generates a token for a password reset link '''
        token = uuid.uuid4()
        self.password_reset = token.hex
        return token.hex

    def __str__(self):
        ''' string representation '''
        return '{!s}'.format(self.user_name)

