''' initializes a new data base and migrates old data '''

import os
import sys
import transaction
import yaml

from datetime import datetime
from tqdm import tqdm
from urllib.parse import urlparse

from pyramid.paster import get_appsettings, setup_logging
from pyramid.scripts.common import parse_vars

from ordr2.models.meta import Base
from ordr2.models import (
    get_engine,
    get_session_factory,
    get_tm_session,
    )
from ordr2.models import Category, Consumable, Order, OrderStatus, User, Role


def usage(argv):
    ''' display the command line help message '''
    cmd = os.path.basename(argv[0])
    print('usage: %s <config_uri> [var=value]\n'
          '(example: "%s development.ini")' % (cmd, cmd))
    sys.exit(1)


def read_yaml_file(*paths):
    ''' read and parse a yaml file '''
    path = os.path.join(*paths)
    with open(path, 'r') as filehandle:
        items = yaml.load(filehandle)
    return items


def parse_dt(value, default=False):
    ''' parse a datetime value or return a default

    for a default value Null is explicitly allowed.
    '''
    if not value or value.startswith('0000'):
        return datetime.utcnow() if default is False else default
    return datetime.strptime(value + ' -0000', '%Y-%m-%d %H:%M:%S %z')


def main(argv=sys.argv):
    ''' setup the database and migrate the data '''
    # load the settings from the provided config file
    if len(argv) < 2:
        usage(argv)
    config_uri = argv[1]
    options = parse_vars(argv[2:])
    setup_logging(config_uri)
    settings = get_appsettings(config_uri, options=options)

    # remove an existing sqlite database to issue a restart
    database_url = urlparse(settings['sqlalchemy.url'])
    if database_url.scheme == 'sqlite' and os.path.isfile(database_url.path):
        os.remove(database_url.path)

    # setup the database connection
    engine = get_engine(settings)
    Base.metadata.create_all(engine)
    session_factory = get_session_factory(engine)

    with transaction.manager:
        dbsession = get_tm_session(session_factory, transaction.manager)
        base_dir = os.path.dirname(__file__)

        # migrate user data
        user_list = read_yaml_file(base_dir, 'export_users.yml')
        for data in tqdm(user_list, desc='Users'):
            user = User(
                id=data['id'],
                user_name=data['username'],
                first_name=data['first_name'],
                last_name=data['last_name'],
                email=data['email'],
                role=Role(data['role'].lower())
            )
            if user.user_name == 'HolgerFrey':
                user.set_password('holgi')
            else:
                user.set_password(data['password'])
            dbsession.add(user)
        dbsession.flush()

        # migrate consumables
        con_list = read_yaml_file(base_dir, 'export_consumables.yml')
        for data in tqdm(con_list, desc='Consumables'):
            category = Category[data['category'].upper()]
            date_created = parse_dt(data['date_created'])
            consumable = Consumable(
                id=data['id'],
                vendor=data['vendor'],
                catalog_nr=data['catalog_number'],
                cas_description=data['CAS_description'],
                package_size=data['package_size'],
                unit_price=data['price_unit'],
                currency=data['currency'],
                comment=data['comment'],
                date_created=date_created,
                date_modified=parse_dt(data['date_modified'], date_created),
                category=category
                )
            dbsession.add(consumable)
        dbsession.flush()

        # migrate orders
        order_list = read_yaml_file(base_dir, 'export_orders.yml')
        for data in tqdm(order_list, desc='Orders'):
            work_status = data.get('work_status', 'approval')
            status = OrderStatus[work_status.upper()]
            category = Category[data['category'].upper()]
            date_created = parse_dt(data['date_created'])
            order = Order(
                id=data['id'],
                vendor=data['vendor'],
                catalog_nr=data['catalog_number'],
                cas_description=data['CAS_description'],
                package_size=data['package_size'],
                unit_price=data['price_unit'],
                amount=data['quantity'],
                total_price=data['price_total'],
                account=data['account'],
                currency=data['currency'],
                comment=data['comment'],
                created_date=date_created,
                created_by=data['username'],
                approval_date=parse_dt(data['date_modified'], None),
                approval_by='',
                ordered_date=parse_dt(data['date_ordered'], None),
                ordered_by='',
                completed_date=parse_dt(data['date_completed'], None),
                completed_by='',
                category=category,
                status=status
                )
            dbsession.add(order)
        dbsession.flush()
