''' views for creating and editing orders '''

import deform
import io
import xlsxwriter

from collections import OrderedDict
from datetime import datetime

from pyramid.httpexceptions import HTTPFound
from pyramid.renderers import render
from pyramid.response import FileIter
from pyramid.view import view_config

from ordr2.events import OrderStatusChange
from ordr2.models import Category, Consumable, Order, OrderStatus, User
from ordr2.schemas.orders import NewOrderSchema, EditOrderSchema

from . import update_column_display


# helper method

def change_in_order_status(request, order, old):
    ''' notifies a user if a noteworthy change in a order occured '''
    noteworthy = False

    if order.status == OrderStatus.APPROVAL and order.status != old:
        order.approval_date = datetime.utcnow()
        order.approval_by = request.user.user_name
    elif order.status == OrderStatus.ORDERED and order.status != old:
        noteworthy = True
        order.ordered_date = datetime.utcnow()
        order.ordered_by = request.user.user_name
    elif order.status == OrderStatus.COMPLETED and order.status != old:
        noteworthy = True
        order.completed_date = datetime.utcnow()
        order.completed_by = request.user.user_name

    if noteworthy and order.created_by != request.user.user_name:
        # only notify if the user who changed the order is not the one who
        # created it in the first place
        account = request.dbsession.\
            query(User).\
            filter_by(user_name=order.created_by).\
            first()
        if account:
            event = OrderStatusChange(request, account, order)
            request.registry.notify(event)


@view_config(
    context='ordr2:resources.OrderList',
    name='view',
    permission='view'
    )
def old_list_redirect(context, request):
    ''' redirect the old /orders/view/ path to /orders/ '''
    return HTTPFound(context.url())


# oder list

@view_config(
    context='ordr2:resources.OrderList',
    permission='view',
    renderer='ordr2:templates/orders/list.jinja2'
    )
def order_list(context, request):
    ''' display the order list '''
    orders = context.items()
    stati = [(s.value.lower(), s.value.capitalize()) for s in OrderStatus]
    return {'orders':orders, 'stati':stati}


@view_config(
    context='ordr2:resources.OrderList',
    name = 'changeview',
    permission='view',
    request_method='POST'
    )
def change_column_view(context, request):
    ''' changes the columns to display '''
    update_column_display(request, 'orders')
    return HTTPFound(context.url())


@view_config(
    context='ordr2:resources.OrderList',
    name='actions',
    request_param='action=export',
    permission='view',
    request_method='POST',
    renderer='ordr2:templates/orders/edit_multiple_stati.jinja2'
    )
def download_view(context, request):
    ''' downloads the displayed order list as an excel file

    see https://xlsxwriter.readthedocs.io/example_http_server3.html '''
    # Create an in-memory output file for the new workbook.
    output = io.BytesIO()
    # Even though the final file will be in memory the module uses temp
    # files during assembly for efficiency. To avoid this on servers that
    # don't allow temp files, for example the Google APP Engine, set the
    # 'in_memory' constructor option to True:
    workbook = xlsxwriter.Workbook(output, {'in_memory': True})
    worksheet = workbook.add_worksheet()

    # formatting
    bold = workbook.add_format({'bold': 1})
    # Add a number format for cells with money.
    money_format = workbook.add_format({'num_format': '0.00'})
    # Add an Excel date format.
    date_format = workbook.add_format({'num_format': 'yy-mm-dd hh:mm'})

    # Write the column headers
    headers = [
        'Placed On', 'CAS / Description', 'Vendor', 'Catalog Nr',
        'Package Size', 'Unit Price', 'Quantity', 'Total Price', 'Currency',
        'Account', 'Status', 'Category', 'Placed By'
        ]
    for col, header in enumerate(headers):
        worksheet.write(0, col, header, bold)

    for row, resource in enumerate(context.items()):
        order = resource.model
        worksheet.write(row + 1, 0, order.created_date, date_format)
        worksheet.write_string(row + 1, 1, order.cas_description)
        worksheet.write_string(row + 1, 2, order.vendor)
        worksheet.write_string(row + 1, 3, order.catalog_nr)
        worksheet.write_string(row + 1, 4, order.package_size)
        worksheet.write(row + 1, 5, order.unit_price, money_format)
        worksheet.write(row + 1, 6, order.amount)
        worksheet.write(row + 1, 7, order.total_price, money_format)
        worksheet.write_string(row + 1, 8, order.currency)
        worksheet.write_string(row + 1, 9, order.account)
        worksheet.write_string(row + 1, 10, order.status.value.capitalize())
        worksheet.write_string(row + 1, 11, order.category.value.capitalize())
        worksheet.write_string(row + 1, 12, order.created_by)

    # Close the workbook before streaming the data.
    workbook.close()
    # Rewind the buffer.
    output.seek(0)

    response = request.response
    response.app_iter = FileIter(output)
    headers = response.headers
    headers['Content-Type'] = 'application/download'
    headers['Accept-Ranges'] = 'bite'
    headers['Content-Disposition'] = 'attachment;filename=order-list.xlsx'
    return response


@view_config(
    context='ordr2:resources.OrderList',
    name='actions',
    request_param='action=search',
    permission='view',
    request_method='POST'
    )
def search(context, request):
    ''' process the search order form '''
    term = request.POST.get('search', '')
    term = term.strip()
    if term:
        return HTTPFound(context.url(search=term, user=None, status=None, p=1))
    return HTTPFound(context.url())


@view_config(
    context='ordr2:resources.OrderList',
    name='actions',
    request_param='action=status',
    permission='edit',
    request_method='POST',
    renderer='ordr2:templates/orders/edit_multiple_stati.jinja2'
    )
def edit_multiple_stati_form(context, request):
    ''' form for editing the stati of multiple orders '''
    order_ids = [v for k, v in request.POST.items() if k == 'marked']
    orders = request.dbsession.\
                     query(Order).\
                     filter(Order.id.in_(order_ids)).\
                     order_by(Order.created_date).\
                     all()
    if len(orders) == 0:
        return HTTPFound(context.url())
    stati = [(s.name, s.value.capitalize()) for s in OrderStatus]
    return {'orders': orders, 'stati': stati}


@view_config(
    context='ordr2:resources.OrderList',
    name='stati',
    permission='edit',
    request_method='POST'
    )
def edit_multiple_stati_form_processing(context, request):
    ''' change the stati of multiple orders '''

    if 'change' in request.POST:
        count = 0
        for key, value in request.POST.items():
            if not key.startswith('order-'):
                continue
            _, order_id = key.split('-', 1)
            order = request.dbsession.query(Order).get(order_id)
            if order:
                old_status = order.status
                try:
                    order.status = OrderStatus[value]
                except ValueError:
                    pass
                if old_status != order.status:
                   change_in_order_status(request, order, old_status)
                count += 1

        if count == 1:
            request.flash('success', 'One order was updated')
        elif count > 1:
            msg = '{} orders were updated.'.format(count)
            request.flash('success', msg)

    return HTTPFound(context.url())


@view_config(
    context='ordr2:resources.OrderList',
    name='actions',
    request_param='action=delete',
    permission='delete',
    request_method='POST',
    renderer='ordr2:templates/orders/delete.jinja2'
    )
def delete_multiple_orders_form(context, request):
    ''' show confirmation page for deleting multiple orders '''
    order_ids = [v for k, v in request.POST.items() if k == 'marked']
    orders = request.dbsession.\
                     query(Order).\
                     filter(Order.id.in_(order_ids)).\
                     order_by(Order.created_date).\
                     all()
    if len(orders) == 0:
        return HTTPFound(context.url())
    return {'orders': orders}


@view_config(
    context='ordr2:resources.OrderList',
    name='delete',
    permission='delete',
    request_method='POST'
    )
@view_config(
    context='ordr2:resources.OrderResource',
    name='delete',
    permission='delete',
    request_method='POST'
    )
def order_delete_form_processing(context, request):
    ''' delete one or multiple orders after confirmation '''
    if 'delete' in request.POST:
        order_ids = [v for k, v in request.POST.items() if k == 'order']
        orders = request.dbsession.\
                        query(Order).\
                        filter(Order.id.in_(order_ids)).\
                        all()
        for order in orders:
            request.dbsession.delete(order)

        if len(orders) == 1:
            request.flash('success', 'One order was deleted')
        elif len(orders) > 1:
            msg = '{} orders were deleted.'.format(len(orders))
            request.flash('success', msg)

    return HTTPFound(request.resource_url(request.root, 'orders'))


# Single Order views

@view_config(
    context='ordr2:resources.OrderResource',
    permission='view',
    request_method='GET',
    renderer='ordr2:templates/orders/view.jinja2'
    )
def order_view(context, request):
    ''' show the order information '''
    return {}


@view_config(
    context='ordr2:resources.OrderResource',
    name='delete',
    permission='delete',
    request_method='GET',
    renderer='ordr2:templates/orders/delete.jinja2'
    )
def order_delete_form(context, request):
    ''' show the confirmation page for deleting one order '''
    return {'orders': [context.model]}


@view_config(
    context='ordr2:resources.OrderResource',
    name='edit',
    permission='edit',
    request_method='GET',
    renderer='ordr2:templates/orders/edit.jinja2'
    )
def order_edit_form(context, request):
    ''' show the edit order form '''
    form = EditOrderSchema.as_form(request)
    order = context.model
    info = {
        'status': order.status.name
        }
    item = {
        'cas_description': order.cas_description,
        'category': order.category.name,
        'vendor': order.vendor,
        'catalog_nr': order.catalog_nr,
        'package_size': order.package_size
        }
    pricing = {
        'unit_price': {
            'amount': '%.2f' % order.unit_price,
            'currency': order.currency
            },
        'quantity': order.amount,
        'total_price': {
            'amount': '%.2f' % order.total_price,
            'currency': order.currency
            },
        }
    optional = {
        'account': order.account,
        'comment': order.comment
        }

    form_data = {
        'order_information': info,
        'item_information': item,
        'pricing': pricing,
        'optional_information': optional
        }
    form.set_appstruct(form_data)
    return {'form': form}


@view_config(
    context='ordr2:resources.OrderResource',
    name='edit',
    permission='edit',
    request_method='POST',
    renderer='ordr2:templates/orders/edit.jinja2'
    )
def order_edit_form_processing(context, request):
    ''' process the edit order form '''

    form = EditOrderSchema.as_form(request)
    data = request.POST.items()

    if 'save' in request.POST:
        try:
            appstruct = form.validate(data)
        except deform.ValidationFailure as e:
            return {'form': form}

        # form validation sucessful, change order
        order = context.model
        info = appstruct['order_information']
        item = appstruct['item_information']
        pricing = appstruct['pricing']
        optional = appstruct['optional_information']
        old_status = order.status

        order.status = OrderStatus[info['status']]

        order.cas_description = item['cas_description']
        order.category = item['category']
        order.vendor = item['vendor']
        order.catalog_nr = item['catalog_nr']
        order.package_size = item['package_size']

        order.unit_price = pricing['unit_price']['amount']
        order.currency = pricing['unit_price']['currency']
        order.amount = pricing['quantity']
        order.total_price = order.unit_price * order.amount

        order.account = optional['account']
        order.comment = optional['comment']

        if old_status != order.status:
           change_in_order_status(request, order, old_status)

        msg = 'Order <em>{!s}</em> updated.'.format(context.model)
        request.flash('success', msg)

    elif 'delete' in request.POST and context.model:
        # redirect to delete order confirmation page
        return HTTPFound(request.resource_url(context, 'delete'))

    elif 'reorder' in request.POST and context.model:
        # redirect to create new order form
        return HTTPFound(
            request.resource_url(
                context.__parent__,
                'new',
                query={'reorder': context.model.id}
                )
            )

    return HTTPFound(context.__parent__.url())


@view_config(
    context='ordr2:resources.OrderList',
    name='new',
    permission='create',
    request_method='GET',
    renderer='ordr2:templates/orders/new.jinja2'
    )
def order_new_form(context, request):
    ''' create a new order '''
    form = NewOrderSchema.as_form(request)

    # check if the form should be prefilled with some data,
    # either from selecting a consumable or from reordering an item
    consumable_id = request.GET.get('consumable', None)
    order_id = request.GET.get('reorder', None)
    prefill = None

    if order_id:
        prefill = request.dbsession.query(Order).get(order_id)
    elif consumable_id:
        prefill = request.dbsession.query(Consumable).get(consumable_id)

    # prefill the form data
    if prefill:
        # some fields depend on if the prefill was a reorder or consumable
        quantity = prefill.amount if order_id else 1
        total_price = prefill.total_price if order_id else prefill.unit_price
        account = prefill.account if order_id else ''
        item = {
            'cas_description': prefill.cas_description,
            'category': prefill.category.name,
            'vendor': prefill.vendor,
            'catalog_nr': prefill.catalog_nr,
            'package_size': prefill.package_size
            }
        pricing = {
            'unit_price': {
                'amount': '%.2f' % prefill.unit_price,
                'currency': prefill.currency
                },
            'quantity': quantity,
            'total_price': {
                'amount': '%.2f' % total_price,
                'currency': prefill.currency
                },
            }
        optional = {
            'account': account,
            'comment': prefill.comment
            }

        form_data = {
            'item_information': item,
            'pricing': pricing,
            'optional_information': optional
            }
        form.set_appstruct(form_data)

    return {'form': form}


@view_config(
    context='ordr2:resources.OrderList',
    name='splash',
    permission='create',
    request_method='GET',
    renderer='ordr2:templates/orders/splash.jinja2'
    )
def order_splash(context, request):
    ''' splash screen for a new order where consumables can be selected '''
    structured = OrderedDict()
    for cat in Category:
        structured[cat] = []
    available = request.dbsession.\
        query(Consumable).\
        order_by(Consumable.cas_description).\
        all()
    names = []
    for consumable in available:
        names.append(consumable.cas_description)
        structured[consumable.category].append(consumable)
    return {'consumable_names': names, 'consumables': structured}


@view_config(
    context='ordr2:resources.OrderList',
    name='splash',
    permission='create',
    request_method='POST',
    renderer='ordr2:templates/orders/splash.jinja2'
    )
def order_splash_processing(context, request):
    ''' process the splash screen selection '''
    name = request.POST.get('search')
    consumable = request.dbsession.\
        query(Consumable).\
        filter_by(cas_description=name).\
        first()
    if consumable:
        return HTTPFound(
            request.resource_url(
                context,
                'new',
                query={'consumable': consumable.id}
                )
            )

    request.flash('error', 'No consumable selected')
    return HTTPFound(request.resource_url(context, 'splash'))


@view_config(
    context='ordr2:resources.OrderList',
    name='new',
    permission='create',
    request_method='POST',
    renderer='ordr2:templates/orders/new.jinja2'
    )
def order_new_form_processing(context, request):
    ''' process the new order form '''

    form = NewOrderSchema.as_form(request)
    data = request.POST.items()

    if 'save' in request.POST:

        try:
            appstruct = form.validate(data)
        except deform.ValidationFailure as e:
            return {'form': form}

        # form validation sucessful, change order
        order = Order(
            status=OrderStatus.OPEN,
            created_by=request.user.user_name
            )
        item = appstruct['item_information']
        pricing = appstruct['pricing']
        optional = appstruct['optional_information']

        order.cas_description = item['cas_description']
        order.category = item['category']
        order.vendor = item['vendor']
        order.catalog_nr = item['catalog_nr']
        order.package_size = item['package_size']

        order.unit_price = pricing['unit_price']['amount']
        order.currency = pricing['unit_price']['currency']
        order.amount = pricing['quantity']
        order.total_price = order.unit_price * order.amount

        order.account = optional['account']
        order.comment = optional['comment']

        request.dbsession.add(order)

        msg = 'Order <em>{!s}</em> created.'.format(context.model)
        request.flash('success', msg)

    return HTTPFound(context.url())

